#include "stdafx.h"
#include "VSGraphXSample.h"
#include "EntitiesBar.h"
#include "VSGraphXSampleView.h"
#include "VSGraphXSampleDoc.h"
#include "MainFrm.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

CEntitiesBar::CEntitiesBar( CWnd* pParent /*=NULL*/ ) : CDialogBar()
{
	//{{AFX_DATA_INIT(CEntitiesBar)
	m_width = _T("");
	m_height = _T("");
	m_otherInfo = _T("");
	//}}AFX_DATA_INIT
}

void CEntitiesBar::DoDataExchange( CDataExchange* pDX )
{
	CDialogBar::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CEntitiesBar)
	DDX_Control( pDX, IDC_LIST, m_list);
	DDX_Text(pDX, IDC_ST_WIDTH, m_width);
	DDX_Text(pDX, IDC_ST_HEIGHT, m_height);
	DDX_Text(pDX, IDC_EDIT_INFO, m_otherInfo);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP( CEntitiesBar, CDialogBar )
	//{{AFX_MSG_MAP(CEntitiesBar)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CEntitiesBar message handlers


BOOL CEntitiesBar::Create( CWnd* pParentWnd )
{
	if( CDialogBar::Create( pParentWnd, IDD_ENTITIES_BAR, CBRS_RIGHT|CBRS_TOOLTIPS|CBRS_FLYBY, IDD_ENTITIES_BAR ) == FALSE )
		return FALSE;

	UpdateData(FALSE);
	return TRUE;
}

//Updates the list control of the entities bar based on a CVSGraphX drawing.
void CEntitiesBar::UpdateList(CVSGraphX *graphx)
{
	m_list.UpdateList(graphx);
}

//Initializes the entity information displayed at the bottom of the bar.
void CEntitiesBar::InitInfo()
{
	m_width = m_height = m_otherInfo = _T("");
	GetDlgItem(IDC_EDIT_INFO)->ShowWindow(SW_HIDE);
	GetDlgItem(IDC_ST_WIDTH_LABEL)->ShowWindow(SW_HIDE);
	GetDlgItem(IDC_ST_HEIGHT_LABEL)->ShowWindow(SW_HIDE);
	UpdateData(FALSE);
}

//Updates the entity information displayed at the bottom of the bar based on the
//current selected entity.
void CEntitiesBar::UpdateInfo(int selection)
{
	CMainFrame *pMainFrm = (CMainFrame *) AfxGetMainWnd();
	if (!pMainFrm)
	{
		InitInfo();
		return;
	}

	CVSGraphXSampleView *pView = (CVSGraphXSampleView *) pMainFrm->GetActiveView();
	if (!pView)
	{
		InitInfo();
		return;
	}

	CVSGraphXSampleDoc *pDoc = (CVSGraphXSampleDoc *) pView->GetDocument();
	if (!pDoc)
	{
		InitInfo();
		return;
	}

	CVSGraphX *graphX = pDoc->m_graphx;
	if (!graphX)
	{
		InitInfo();
		return;
	}

	if (selection < 0 || selection > graphX->GetNumOfEntities() - 1)
	{
		InitInfo();
		return;
	}

	CString unitsStr = pView->GetUnitsString();

	double w = graphX->GetEntityWidth(selection);
	double h = graphX->GetEntityHeight(selection);

	m_width.Format(_T("%.3f %s"), w, unitsStr);
	m_height.Format(_T("%.3f %s"), h, unitsStr);

	GetDlgItem(IDC_ST_WIDTH_LABEL)->ShowWindow(SW_SHOW);
	GetDlgItem(IDC_ST_HEIGHT_LABEL)->ShowWindow(SW_SHOW);

	int type = graphX->GetEntityType(selection);
	if (type == VSGRAPHX_ENTITY_POLYBEZIER)
	{
		double pnWidth = 0.;
		UINT pnStyle = 0, brStyle = 0;
		COLORREF pnColor = 0, brColor = 0;
		LONG hatch = 0;
		graphX->GetPolyBezierPathInfo(selection, &pnWidth, &pnStyle, &pnColor, 
									  &brStyle, &brColor, &hatch);

		m_otherInfo.Format(_T("Pen width: %.3f %s"), pnWidth, unitsStr);
		GetDlgItem(IDC_EDIT_INFO)->ShowWindow(SW_SHOW);
	}
	else if (type == VSGRAPHX_ENTITY_TEXT)
	{
		m_otherInfo = _T("");
		int length = graphX->GetFontName(selection, NULL, 0);
		if (length > 0)
		{
			LPTSTR lpFontName = new TCHAR[length+1];
			lpFontName[length] = _T('\0');
			int result = graphX->GetFontName(selection, lpFontName, length);
			if (result != VSGRAPHX_ERROR)
			{
				m_otherInfo += _T("Font: ");
				m_otherInfo += lpFontName;
				m_otherInfo += _T("\r\n");;
			}

			delete lpFontName;
		}

		double poX = 0., poY = 0., textHeight = 0., angle = 0.;
		BYTE style = 0;
		COLORREF color = 0;
		graphX->GetTextInfo(selection, &poX, &poY, &style, &textHeight, &angle, &color);

		CString styleStr = _T("Style: ");
		if (style & VSGRAPHX_TEXT_STYLE_BOLD)
			styleStr += _T("Bold ");
		if (style & VSGRAPHX_TEXT_STYLE_ITALIC)
			styleStr += _T("Italic ");
		if (style & VSGRAPHX_TEXT_STYLE_UNDERLINE)
			styleStr += _T("Underline");

		if (styleStr.GetLength() > 7)
			styleStr += _T("\r\n");
		else
			styleStr += _T("Normal\r\n");

		m_otherInfo += styleStr;

		CString str;
		str.Format(_T("Text height: %.3f %s\r\nAngle: %.1f degrees"), textHeight, unitsStr, angle);
		m_otherInfo += str;

		GetDlgItem(IDC_EDIT_INFO)->ShowWindow(SW_SHOW);
	}
	else
	{
		m_otherInfo = _T("");
		GetDlgItem(IDC_EDIT_INFO)->ShowWindow(SW_HIDE);
	}

	UpdateData(FALSE);
}

