// VSGraphXSampleView.cpp : implementation of the CVSGraphXSampleView class
//

#include "stdafx.h"
#include "VSGraphXSample.h"

#include "VSGraphXSampleDoc.h"
#include "VSGraphXSampleView.h"
#include "MainFrm.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CVSGraphXSampleView

IMPLEMENT_DYNCREATE(CVSGraphXSampleView, CView)

BEGIN_MESSAGE_MAP(CVSGraphXSampleView, CView)
	//{{AFX_MSG_MAP(CVSGraphXSampleView)
	ON_WM_HSCROLL()
	ON_WM_VSCROLL()
	ON_WM_KEYDOWN()
	ON_WM_ERASEBKGND()
	ON_COMMAND(ID_ZOOM_FIT, OnZoomFit)
	ON_COMMAND(ID_ZOOM_IN, OnZoomIn)
	ON_COMMAND(ID_ZOOM_OUT, OnZoomOut)
	ON_COMMAND(ID_UNITS_CENTIMETERS, OnUnitsCentimeters)
	ON_UPDATE_COMMAND_UI(ID_UNITS_CENTIMETERS, OnUpdateUnitsCentimeters)
	ON_COMMAND(ID_UNITS_INCHES, OnUnitsInches)
	ON_UPDATE_COMMAND_UI(ID_UNITS_INCHES, OnUpdateUnitsInches)
	ON_COMMAND(ID_UNITS_MILLIMETERS, OnUnitsMillimeters)
	ON_UPDATE_COMMAND_UI(ID_UNITS_MILLIMETERS, OnUpdateUnitsMillimeters)
	ON_COMMAND(ID_UNITS_TENTHSOFMILLIMETERS, OnUnitsTenthsOfMillimeters)
	ON_UPDATE_COMMAND_UI(ID_UNITS_TENTHSOFMILLIMETERS, OnUpdateUnitsTenthsOfMillimeters)
	ON_COMMAND(ID_UNITS_PIXELS, OnUnitsPixels)
	ON_UPDATE_COMMAND_UI(ID_UNITS_PIXELS, OnUpdateUnitsPixels)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CVSGraphXSampleView construction/destruction

CVSGraphXSampleView::CVSGraphXSampleView()
{
	m_zoomFactor = 1.;
	m_leftSide = m_topSide = 0.;
	m_selection = -1;

	m_units = VSGRAPHX_UNITS_MM10;
	m_unitsFactor = 1.;
}

CVSGraphXSampleView::~CVSGraphXSampleView()
{
}

BOOL CVSGraphXSampleView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CView::PreCreateWindow(cs);
}


/////////////////////////////////////////////////////////////////////////////
// CVSGraphXSampleView drawing

//Conversion from double to integer
static int d2i(double d)
{
	if (d>=0.)
		return (int)(d+.5);
	else
		return (int)(d-.5);
}

//Handled by OnDraw
BOOL CVSGraphXSampleView::OnEraseBkgnd(CDC* pDC) 
{
	return TRUE;
}

void CVSGraphXSampleView::OnInitialUpdate()
{
	SetScrollRange(SB_HORZ, -20000, 20000);
	SetScrollRange(SB_VERT, -20000, 20000);
	SetScrollPos(SB_HORZ, 0);
	SetScrollPos(SB_VERT, 0);

	CView::OnInitialUpdate();
}


void CVSGraphXSampleView::OnDraw(CDC* pDC)
{
	CVSGraphXSampleDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	CRect viewRect;
	GetClientRect(&viewRect);

	//Create a 24-bit empty bitmap having the dimensions of the view
	BITMAPINFOHEADER bi;
	memset(&bi,0,sizeof(bi));

	bi.biSize			= sizeof(bi);
	bi.biWidth			= viewRect.Width();
	bi.biHeight			= viewRect.Height();
	bi.biBitCount		= 24;
	bi.biPlanes			= 1;
	bi.biCompression    = BI_RGB;

	BYTE *bits=NULL;

	HBITMAP hBmp = CreateDIBSection(NULL,(BITMAPINFO*)&bi,DIB_RGB_COLORS,(void**)&bits,NULL,0);
	if(hBmp==NULL)
		return;

	//Create a compatible dc having the bitmap selected and copy it to the normal dc.
	CDC cdc;
	cdc.CreateCompatibleDC(pDC);
	CBitmap *oldbmp = cdc.SelectObject(CBitmap::FromHandle(hBmp));

	CDC *pDCSave = pDC;
	pDC = &cdc;

	//Fill the view rectangle with white
	CBrush bkbrush(RGB(255,255,255));
	pDC->FillRect(viewRect,&bkbrush);

	if (pDoc->m_graphx && m_zoomFactor >= 0.00001 && m_zoomFactor <= 100000.)
	{
		//Drawing width and height taking the zoom factor into consideration
		double width = pDoc->m_graphx->GetWidth() * m_zoomFactor;
		double height = pDoc->m_graphx->GetHeight() * m_zoomFactor;

		//Assign a clip region to pDC
		CRgn rgn;
		rgn.CreateRectRgn(viewRect.left, viewRect.top, viewRect.right, viewRect.bottom);
		pDC->SelectClipRgn(&rgn, RGN_COPY);

		//The rectangle where the whole drawing should be drawn.
		CRect drawingRect(d2i(m_leftSide), d2i(m_topSide), 
						  d2i(m_leftSide + width), d2i(m_topSide + height));

		int scrollPosX = GetScrollPos(SB_HORZ), scrollPosY = GetScrollPos(SB_VERT);
		drawingRect.OffsetRect(-scrollPosX, -scrollPosY);

		//The intersection of the drawing rectangle and the view rectangle
		CRect interRect;
		if (drawingRect.left > viewRect.right || drawingRect.right < viewRect.left ||
			drawingRect.top > viewRect.bottom || drawingRect.bottom < viewRect.top)
		{
			interRect.SetRectEmpty();
		}
		else
		{
			interRect.left = max(drawingRect.left, viewRect.left);
			interRect.right = min(drawingRect.right, viewRect.right);
			interRect.top = max(drawingRect.top, viewRect.top);
			interRect.bottom = min(drawingRect.bottom, viewRect.bottom);
		}

		//The whole drawing can be drawn inside the view rectangle
		if (interRect == drawingRect)
			pDoc->m_graphx->Draw(*pDC,drawingRect);

		//Only an area of the drawing can be drawn inside the view rectangle.
		else if (!interRect.IsRectNull())
		{
			double l = 0., t = 0., r = 0., b = 0.;
			pDoc->m_graphx->GetBounds(&l, &t, &r, &b);

			double factorW = pDoc->m_graphx->GetWidth() / max(drawingRect.Width(),0.000001);
			double factorH = pDoc->m_graphx->GetHeight() / max(drawingRect.Height(),0.000001);

			double right = l + (interRect.right - drawingRect.left) * factorW;
			double left = l + (interRect.left - drawingRect.left) * factorW;
			double bottom = t + (interRect.bottom - drawingRect.top) * factorH;
			double top = t + (interRect.top - drawingRect.top) * factorH;

			pDoc->m_graphx->DrawArea(*pDC,interRect,left,top,right,bottom);
		}

		pDC->SelectClipRgn(NULL, RGN_COPY);
		rgn.DeleteObject();

		//Draw the selection rectangle if any.
		DrawSelectionRect(pDC);
	}

	//Copy the bitmap of the compatible dc to the normal dc.
	pDC = pDCSave;
	pDC->BitBlt(0,0,viewRect.Width(),viewRect.Height(),&cdc,0,0,SRCCOPY);
	cdc.SelectObject(oldbmp);
	cdc.DeleteDC();
	DeleteObject(hBmp);
}


//Draw the selection rectangle if any.
void CVSGraphXSampleView::DrawSelectionRect(CDC *pDC)
{
	if (!pDC)
		return;

	CVSGraphXSampleDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	if (!pDoc->m_graphx)
		return;

	if (m_selection < 0 || m_selection >= pDoc->m_graphx->GetNumOfEntities())
		return;

	double l = pDoc->m_graphx->GetEntityLeft(m_selection);
	double t = pDoc->m_graphx->GetEntityTop(m_selection);
	l = (l - pDoc->m_graphx->GetLeft()) * m_zoomFactor + m_leftSide;
	t = (t - pDoc->m_graphx->GetTop()) * m_zoomFactor + m_topSide;

	double w = pDoc->m_graphx->GetEntityWidth(m_selection) * m_zoomFactor;
	double h = pDoc->m_graphx->GetEntityHeight(m_selection) * m_zoomFactor;

	CRect selRect(d2i(l), d2i(t), d2i(l + w), d2i(t + h));

	int scrollPosX = GetScrollPos(SB_HORZ), scrollPosY = GetScrollPos(SB_VERT);
	selRect.OffsetRect(-scrollPosX, -scrollPosY);

	int oldRop2 = pDC->SetROP2(R2_NOTXORPEN);
	CPen pen(PS_SOLID, 3, RGB(0, 0, 255));
	CPen *pOldPen = pDC->SelectObject(&pen);
	CBrush *pOldBrush = pDC->SelectObject(CBrush::FromHandle((HBRUSH)GetStockObject(NULL_BRUSH)));

	if (selRect.Width() > 0. && selRect.Height() > 0.)
		pDC->Rectangle(&selRect);
	else
	{
		pDC->MoveTo(selRect.TopLeft());
		pDC->LineTo(selRect.BottomRight());
	}

	pDC->SelectObject(pOldBrush);
	pDC->SelectObject(pOldPen);
	pDC->SetROP2(oldRop2);
}


/////////////////////////////////////////////////////////////////////////////
// CVSGraphXSampleView printing

BOOL CVSGraphXSampleView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CVSGraphXSampleView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add extra initialization before printing
}

void CVSGraphXSampleView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}

/////////////////////////////////////////////////////////////////////////////
// CVSGraphXSampleView diagnostics

#ifdef _DEBUG
void CVSGraphXSampleView::AssertValid() const
{
	CView::AssertValid();
}

void CVSGraphXSampleView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CVSGraphXSampleDoc* CVSGraphXSampleView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CVSGraphXSampleDoc)));
	return (CVSGraphXSampleDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////


//Updates the positions of the scrollbars
void CVSGraphXSampleView::UpdateScrollBars(int x, int y)
{
	SetScrollPos(SB_HORZ, x);
	SetScrollPos(SB_VERT, y);
}

//Sets the drawing's selected entity and invalidates.
void CVSGraphXSampleView::SetSelection(int selection)
{
	if (m_selection != selection)
	{
		m_selection = selection;
		Invalidate();
	}
}

//Gets the units used in the view.
int CVSGraphXSampleView::GetUnits()
{
	return m_units;
}

//Sets the units and the units factor for the view and the current drawing.
//It also updates the zoom factor of the view, the document's title and the
//entities bar.
void CVSGraphXSampleView::SetUnits(int units)
{
	if (units < VSGRAPHX_UNITS_PX || units > VSGRAPHX_UNITS_IN || units == m_units)
		return;

	m_units = units;

	//Calculate new units factor
	double oldUnitsFactor = m_unitsFactor;
	switch (m_units)
	{
	case VSGRAPHX_UNITS_PX:
	{
		HDC ndc = ::GetDC(NULL);
		m_unitsFactor = 254. / double(::GetDeviceCaps(ndc,LOGPIXELSX));
		break;
	}
	case VSGRAPHX_UNITS_MM10:
		m_unitsFactor = 1.;
		break;
	case VSGRAPHX_UNITS_MM:
		m_unitsFactor = 10.;
		break;
	case VSGRAPHX_UNITS_CM:
		m_unitsFactor = 100.;
		break;
	case VSGRAPHX_UNITS_IN:
		m_unitsFactor = 254.;
		break;
	default:
		m_unitsFactor = 1.;
		break;
	}

	CVSGraphXSampleDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	if (!pDoc->m_graphx)
		return;

	//Update the units of the current drawing
	pDoc->m_graphx->SetUnits(m_units);

	//Update the view's zoom factor due to the units change
	m_zoomFactor *= m_unitsFactor / oldUnitsFactor;
	Invalidate();

	//Update document's title
	CString path = pDoc->GetPathName();
	char name[_MAX_PATH], ext[_MAX_PATH];
	_splitpath(path, NULL, NULL, name, ext);

	path = name;
	path += ext;
	pDoc->SetTitle(path);

	//Update the entities bar
	((CMainFrame *) AfxGetMainWnd())->m_entitiesBar.UpdateInfo(m_selection);
}

//Gets a string representation for the units used in the view.
CString CVSGraphXSampleView::GetUnitsString()
{
	switch (m_units)
	{
	case VSGRAPHX_UNITS_PX:
		return _T("px");
	case VSGRAPHX_UNITS_MM10:
		return _T("mm/10");
	case VSGRAPHX_UNITS_MM:
		return _T("mm");
	case VSGRAPHX_UNITS_CM:
		return _T("cm");
	case VSGRAPHX_UNITS_IN:
		return _T("in");
	default:
		return _T("mm/10");
	}
}


/////////////////////////////////////////////////////////////////////////////
// CVSGraphXSampleView message handlers

//Fit window
void CVSGraphXSampleView::OnZoomFit() 
{
	CVSGraphXSampleDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	if (!pDoc->m_graphx)
	{
		UpdateScrollBars(0, 0);
		return;
	}

	CRect viewRect;
	GetClientRect(&viewRect);
	double width = max(pDoc->m_graphx->GetWidth(), 0.000001);
	double height = max(pDoc->m_graphx->GetHeight(), 0.000001);
	double newFactor = min((viewRect.Width()/width), (viewRect.Height()/height));

	int oldPosX = GetScrollPos(SB_HORZ), oldPosY = GetScrollPos(SB_VERT);
	if (m_zoomFactor == newFactor && oldPosX == 0 && oldPosY == 0)
		return;

	m_zoomFactor = newFactor;

	double dif = viewRect.Width() - newFactor * width;
	if (dif > 0.)
		m_leftSide = dif / 2.;
	else
		m_leftSide = 0.;

	dif = viewRect.Height() - newFactor * height;
	if (dif > 0.)
		m_topSide = dif / 2.;
	else
		m_topSide = 0.;

	UpdateScrollBars(0, 0);

	Invalidate();
}

//Zoom in (we use as center the center of the view rectangle).
void CVSGraphXSampleView::OnZoomIn() 
{
	CVSGraphXSampleDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	if (!pDoc->m_graphx)
		return;

	CRect viewRect;
	GetClientRect(&viewRect);
	double centerX = ((double) viewRect.left + (double) viewRect.right) / 2.;
	double centerY = ((double) viewRect.top + (double) viewRect.bottom) / 2.;

	int scrollPosX = GetScrollPos(SB_HORZ), scrollPosY = GetScrollPos(SB_VERT);
	centerX += scrollPosX;
	centerY += scrollPosY;

	//We don't let width or height to be zero.
	double width = max(pDoc->m_graphx->GetWidth(), 0.000001);
	double height = max(pDoc->m_graphx->GetHeight(), 0.000001);

	double newFactor = m_zoomFactor * 1.5;
	if (width * newFactor > 32000. || height * newFactor > 32000.)
		newFactor = min(32000. / width, 32000. / height);

	if (m_zoomFactor == newFactor)
		return;

	m_leftSide = (m_leftSide - centerX)* newFactor / m_zoomFactor + centerX;
	m_topSide = (m_topSide - centerY)* newFactor / m_zoomFactor + centerY;
	m_zoomFactor = newFactor;

	Invalidate();
}

//Zoom out (we use as center the center of the view rectangle).
void CVSGraphXSampleView::OnZoomOut() 
{
	CVSGraphXSampleDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	if (!pDoc->m_graphx)
		return;

	CRect viewRect;
	GetClientRect(&viewRect);
	double centerX = ((double) viewRect.left + (double) viewRect.right) / 2.;
	double centerY = ((double) viewRect.top + (double) viewRect.bottom) / 2.;

	int scrollPosX = GetScrollPos(SB_HORZ), scrollPosY = GetScrollPos(SB_VERT);
	centerX += scrollPosX;
	centerY += scrollPosY;

	//We don't let width or height to be zero.
	double width = max(pDoc->m_graphx->GetWidth(), 0.000001);
	double height = max(pDoc->m_graphx->GetHeight(), 0.000001);

	double newFactor = m_zoomFactor / 1.5;
	if (width * newFactor < 10. && height * newFactor < 10.)
		newFactor = min(10. / width, 10. / height);

	if (m_zoomFactor == newFactor)
		return;

	m_leftSide = (m_leftSide - centerX)* newFactor / m_zoomFactor + centerX;
	m_topSide = (m_topSide - centerY)* newFactor / m_zoomFactor + centerY;
	m_zoomFactor = newFactor;

	Invalidate();
}

//Handle horizontal scrollbar
void CVSGraphXSampleView::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
	int oldPosX = GetScrollPos(SB_HORZ), oldPosY = GetScrollPos(SB_VERT);
	int newPosX = oldPosX;

	CRect rc;
	GetClientRect(rc);

	switch(nSBCode)
	{
	case SB_LINELEFT:
		newPosX -= rc.Width() / 10;
		break;
	case SB_LINERIGHT:
		newPosX += rc.Width() / 10;
		break;
	case SB_PAGELEFT:
		newPosX -= rc.Width() * 2 / 3;
		break;
	case SB_PAGERIGHT:
		newPosX += rc.Width() * 2 / 3;
		break;
	case SB_THUMBPOSITION:
		newPosX = (int) nPos;
		break;
	}

	if (newPosX != oldPosX)
	{
		UpdateScrollBars(newPosX, oldPosY);
		Invalidate();
	}
}

//Handle vertical scrollbar
void CVSGraphXSampleView::OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
	int oldPosX = GetScrollPos(SB_HORZ), oldPosY = GetScrollPos(SB_VERT);
	int newPosY = oldPosY;

	CRect rc;
	GetClientRect(rc);

	switch(nSBCode)
	{
	case SB_LINEUP:
		newPosY -= rc.Height() / 10;
		break;
	case SB_LINEDOWN:
		newPosY += rc.Height() / 10;
		break;
	case SB_PAGEUP:
		newPosY -= rc.Height() * 2 / 3;
		break;
	case SB_PAGEDOWN:
		newPosY += rc.Height() * 2 / 3;
		break;
	case SB_THUMBPOSITION:
		newPosY = (int) nPos;
		break;
	}

	if (newPosY != oldPosY)
	{
		UpdateScrollBars(oldPosX, newPosY);
		Invalidate();
	}
}


void CVSGraphXSampleView::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags)
{
	switch (nChar)
	{
	case VK_LEFT:
		OnHScroll(SB_LINELEFT,0,NULL);
		break;
	case VK_RIGHT:
		OnHScroll(SB_LINERIGHT,0,NULL);
		break;
	case VK_UP:
		OnVScroll(SB_LINEUP,0,NULL);
		break;
	case VK_DOWN:
		OnVScroll(SB_LINEDOWN,0,NULL);
		break;
	case VK_PRIOR:
		OnVScroll(SB_PAGEUP,0,NULL);
		break;
	case VK_NEXT:
		OnVScroll(SB_PAGEDOWN,0,NULL);
		break;
	default:
		break;
	}
}


void CVSGraphXSampleView::OnUnitsCentimeters()
{
	SetUnits(VSGRAPHX_UNITS_CM);
}

void CVSGraphXSampleView::OnUpdateUnitsCentimeters(CCmdUI* pCmdUI)
{
	pCmdUI->SetCheck(m_units == VSGRAPHX_UNITS_CM);
}

void CVSGraphXSampleView::OnUnitsInches()
{
	SetUnits(VSGRAPHX_UNITS_IN);
}

void CVSGraphXSampleView::OnUpdateUnitsInches(CCmdUI* pCmdUI)
{
	pCmdUI->SetCheck(m_units == VSGRAPHX_UNITS_IN);
}

void CVSGraphXSampleView::OnUnitsMillimeters()
{
	SetUnits(VSGRAPHX_UNITS_MM);
}

void CVSGraphXSampleView::OnUpdateUnitsMillimeters(CCmdUI* pCmdUI)
{
	pCmdUI->SetCheck(m_units == VSGRAPHX_UNITS_MM);
}

void CVSGraphXSampleView::OnUnitsTenthsOfMillimeters()
{
	SetUnits(VSGRAPHX_UNITS_MM10);
}

void CVSGraphXSampleView::OnUpdateUnitsTenthsOfMillimeters(CCmdUI* pCmdUI)
{
	pCmdUI->SetCheck(m_units == VSGRAPHX_UNITS_MM10);
}

void CVSGraphXSampleView::OnUnitsPixels()
{
	SetUnits(VSGRAPHX_UNITS_PX);
}

void CVSGraphXSampleView::OnUpdateUnitsPixels(CCmdUI* pCmdUI)
{
	pCmdUI->SetCheck(m_units == VSGRAPHX_UNITS_PX);
}